@@ -10,7 +10,8 @@ from . import pyglass
 
 # Initialize the web service
 pyglass.Service.GetInstance().Init(2, pyglass.ConnectionSettings())
-
+# Initialize the DCMTK Codecs
+pyglass.DICOMReader_RegisterDecoderCodecs()
 
 class Block:
     """
@@ -62,6 +63,13 @@ class Project:
 
     def __init__(self, project_impl):
         self.impl = project_impl
+        voxel_size = self.impl.GetVoxelSize()
+        ivec3_dimensions = self.impl.GetProjectDimensionsInUnitVoxels()
+        vec3_dimensions = pyglass.vec3(ivec3_dimensions.x, ivec3_dimensions.y, ivec3_dimensions.z)
+        frame = pyglass.CoordinateFrame(voxel_size, vec3_dimensions)
+        frame.SetOrigin(pyglass.CoordinateFrame.Center)
+        frame.SetUnits(pyglass.CoordinateFrame.PhysicalUnits)
+        self.impl.SetCoordinateFrame(frame)
 
     def __enter__(self):
         return self
@@ -199,7 +207,7 @@ class Project:
     def get_mask(self, roi_index: int, experiment: str = "default") -> Block:
         """
         Returns a given mask of the project's volumetric data as a ``Block`` object.
-        Currently does not support time series data and only supports 8 bit and 16 bit data types for now.
+        Currently only supports 8 bit and 16 bit data types for now.
 
         :parameter roi_index: the index of which roi mask to be retreived 
         :parameter experiment: the name of the current experiment
@@ -212,8 +220,15 @@ class Project:
             )
 
         roi_dims = self.get_roi_dimensions(roi_index, experiment)
-        raster = self.impl.GetMaskRaster(experiment, pyglass.ivec3(int(roi_dims[2]), int(roi_dims[1]), int(roi_dims[0])), roi_index)
         offset = self.get_roi_offset(roi_index, experiment)
+        resolution_map = self.get_resolution_map()
+        max_resolution_level = len(resolution_map) - 1
+
+        vec3_offset = pyglass.vec3(offset[2], offset[1], offset[0])
+        vec3_dims = pyglass.vec3(float(roi_dims[2]), float(roi_dims[1]), float(roi_dims[0]))
+ 
+        extractor = pyglass.MaskOctreeRasterExtractor(None)
+        raster = extractor.GetCustomBlock(self.impl, 0, max_resolution_level, vec3_offset, vec3_dims)
         return Block(pyglass.GetRasterAsNumpyArray(raster), offset)
 
     @_closed_project_decorator
@@ -897,6 +912,27 @@ class Project:
         self.impl.ImportOnionSeriesCSV(experiment, "temp.csv")
         os.remove("temp.csv")
 
+    @_closed_project_decorator
+    def add_tracking_series(
+        self,
+        multi_points: Mapping[int, np.ndarray],
+        experiment: str = "default"
+    ) -> None:
+        """
+        Imports a single series of tracking points. 
+
+        :param multi_points: Dictionary mapping the timepoint to the tracked position.
+                             Position is in z, y, x order, in physical units, and 
+                             relative to the center of the highest resolution level.
+        :param experiment: the name of the experiment to add the tracking series to
+        """
+        typedSeries = pyglass.IntVec3Map()
+        for point in multi_points:
+            if multi_points[point].size != 3:
+                raise ValueError("Tracking position at timepoint " + str(point) + " is invalid. Positions must have 3 values.")
+            typedSeries[point] = pyglass.vec3(float(multi_points[point][2]), float(multi_points[point][1]), float(multi_points[point][0]))
+        self.impl.AddOnionSeries(experiment, typedSeries)
+
     @_closed_project_decorator
     def import_meshes(
         self, 
@@ -1493,5 +1529,22 @@ def create_project(
 
         data_providers.append(new_data_provider)
 
+def get_presentation(src_path : str) -> pyglass.PresentationFile:
+    """
+    :param src_path: full path to SYP file
+    :return: a ``PresentationFile`` object read from the given file
+    """
+    writer = pyglass.PresentationFileWriter()
+    return writer.ReadFileWorkingDir(pyglass.path(src_path))
+
+def write_presentation(pres_file : pyglass.PresentationFile, dst_path : str) -> None:
+    """
+    :param pres_file: ``PresentationFile`` object to write
+    :param dst_path: full path to output SYP file
+    :return: None
+    """
+    writer = pyglass.PresentationFileWriter()
+    writer.WriteFileWorkingDir(pres_file, pyglass.path(dst_path))
+
 if __name__ == "__main__":
     print("TODO: some info here")
